-------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity vdogen is port(

  nrs       : in  std_logic;
  cl1       : in  std_logic;
  cl2       : in  std_logic;
  key       : in  std_logic;
  repk      : in  std_logic;
  wtrx      : in  std_logic;
  trax      : in  std_logic;
  vpg       : in  std_logic;
  vclk      : in  std_logic;
  bufpixel  : in  std_logic_vector( 7 downto 0);
  vdomem    : out std_logic_vector(14 downto 0);
  nachlad   : out std_logic;
  vibl      : out std_logic;
  vign      : out std_logic;
  vird      : out std_logic;
  vihs      : out std_logic;
  vivs      : out std_logic);

end vdogen;
architecture behvdogen of vdogen is

signal vpgv          : std_logic := '0';
signal ffhs,ffvs     : std_logic := '1';
signal respuls       : std_logic := '0';
signal setpuls       : std_logic;

signal getphas,ladphas,aoutphas,boutphas  : std_logic;
signal swihct,maxhcp,maxhct,maxvct        : std_logic;
signal zphas,repdl,runzeit,accp           : std_logic;
signal zeit,neuzeit,vzeit,xzeit,xneuzeit  : std_logic_vector(31 downto 0);
signal pixel,neupixel,xpix,zpix,opix      : std_logic_vector( 7 downto 0);
signal neuhcp,neuhct,neuvct               : std_logic_vector( 9 downto 0);
signal inx                                : std_logic_vector( 6 downto 0);

signal hcp           : std_logic_vector( 9 downto 0) := "0000001000";
signal hct,vct       : std_logic_vector( 9 downto 0) := "0000000000";

signal memct         : std_logic_vector(13 downto 0) := "00000000000000";

type feld is array(0 to 127) of std_logic_vector(7 downto 0);

constant wert : feld := (x"3c",x"24",x"24",x"24",x"24",x"24",x"3c",x"00",
                         x"04",x"04",x"04",x"04",x"04",x"04",x"04",x"00",
                         x"3c",x"04",x"04",x"3c",x"20",x"20",x"3c",x"00",
                         x"3c",x"04",x"04",x"3c",x"04",x"04",x"3c",x"00",
                         x"24",x"24",x"24",x"3c",x"04",x"04",x"04",x"00",
                         x"3c",x"20",x"20",x"3c",x"04",x"04",x"3c",x"00",
                         x"3c",x"20",x"20",x"3c",x"24",x"24",x"3c",x"00",
                         x"3c",x"04",x"04",x"04",x"04",x"04",x"04",x"00",
                         x"3c",x"24",x"24",x"3c",x"24",x"24",x"3c",x"00",
                         x"3c",x"24",x"24",x"3c",x"04",x"04",x"3c",x"00",
                         x"3c",x"24",x"24",x"3c",x"24",x"24",x"24",x"00",
                         x"3c",x"20",x"20",x"3c",x"24",x"24",x"24",x"00",
                         x"3c",x"20",x"20",x"20",x"20",x"20",x"3c",x"00",
                         x"3c",x"04",x"04",x"3c",x"24",x"24",x"24",x"00",
                         x"3c",x"20",x"20",x"3c",x"20",x"20",x"3c",x"00",
                         x"3c",x"20",x"20",x"3c",x"20",x"20",x"20",x"00");
begin

vihs    <= ffhs;
vivs    <= ffvs;
nachlad <= getphas;

vdomem  <= (not vpgv) & memct(5 downto 0) & memct(13 downto 6);
------------------------------------------------------------------------------------------------------------

neuhcp <= "0000000000" when maxhcp = '1' else std_logic_vector(unsigned(hcp) + 1);
neuhct <= "0000000000" when maxhct = '1' else std_logic_vector(unsigned(hct) + 1);
neuvct <= "0000000000" when maxvct = '1' else std_logic_vector(unsigned(vct) + 1);
------------------------------------------------------------------------------------------------------------

writehcp : process(vclk) begin
if vclk'event and vclk = '1' then if trax = '0' then hcp <= neuhcp;
end if; end if; end process writehcp;

writehct : process(vclk) begin
if vclk'event and vclk = '1' then if trax = '0' then hct <= neuhct;
end if; end if; end process writehct;

writevct : process(vclk) begin
if vclk'event and vclk = '1' then if swihct = '1' then vct <= neuvct;
end if; end if; end process writevct;

swihct <= hct(9) and not hct(8) and     hct(7) and not hct(6) and not hct(5) and not hct(4)
                                and not hct(3) and not hct(2) and not hct(1) and not hct(0);

maxhcp <= hcp(9) and     hcp(8) and not hcp(7) and not hcp(6) and not hcp(5) and     hcp(4)
                                and     hcp(3) and     hcp(2) and     hcp(1) and     hcp(0);

maxhct <= hct(9) and     hct(8) and not hct(7) and not hct(6) and not hct(5) and     hct(4)
                                and     hct(3) and     hct(2) and     hct(1) and     hct(0);

maxvct <= vct(9) and not vct(8) and not vct(7) and not vct(6) and not vct(5) and not vct(4)
                                and     vct(3) and not vct(2) and not vct(1) and not vct(0);
------------------------------------------------------------------------------------------------------------
getphas <= not (hcp(9) or hcp(2) or hcp(1) or hcp(0)) and  not vct(9) and (vct(8) xor vct(7));
ladphas <= not (hct(9) or hct(2) or hct(1) or hct(0)) and  not vct(9) and (vct(8) xor vct(7));

zphas   <= not (hct(2) or hct(1) or hct(0));
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
writepixel : process(vclk) begin
if vclk'event and vclk = '1' then pixel <= neupixel;
end if; end process writepixel;

neupixel <= (pixel(6 downto 0) & '0') when zphas = '0' else not opix;

opix <= bufpixel when ladphas = '1' else zpix;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
writememct  : process(vclk) begin if vclk'event and vclk = '1' then if getphas = '1'
 then memct <= std_logic_vector(unsigned(memct)+1); end if; end if;
 end process writememct;

writevpgv   : process(vclk) begin if vclk'event and vclk = '1' then if ffvs = '0'
 then vpgv <= vpg; end if; end if;
 end process writevpgv;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
 setpuls <= hct(9) and not hct(8) and not hct(7) and     hct(6) and not hct(5) and     hct(4)
                                  and not hct(3) and not hct(2) and not hct(1) and not hct(0);

writerespuls : process(vclk) begin if vclk'event and vclk = '1' then
 respuls <= hct(9) and not hct(8) and     hct(7) and not hct(6) and     hct(5) and     hct(4)
                                  and not hct(3) and not hct(2) and not hct(1) and not hct(0);
 end if; end process writerespuls;
------------------------------------------------------------------------------------------------------------
writeffhs : process(vclk,respuls) begin if respuls = '1' then ffhs <= '1';
        elsif vclk'event and vclk = '1' then if setpuls = '1' then ffhs <= '0';
 end if; end if; end process writeffhs;
------------------------------------------------------------------------------------------------------------
writeffvs : process(vclk) begin
 if vclk'event and vclk = '1' then
 ffvs <= not (not vct(9) and     vct(8) and     vct(7) and     vct(6) and     vct(5)
                         and     vct(4) and not vct(3) and     vct(2) and     vct(1));
 end if; end process writeffvs;

------------------------------------------------------------------------------------------------------------
writeaoutphas : process(vclk) begin if vclk'event and vclk = '1' then
 aoutphas <= (not hct(9) and not vct(9) and (vct(8) xor vct(7))) or
 (not (hct(9) or hct(8) or hct(7) or hct(6)) and vct(8) and vct(7) and not (vct(6) or vct(5)) and
 ((not vct(4) and vct(3) and (vct(2) or (vct(1) and vct(0)))) or (vct(4) and not vct(3) and not vct(2))));
 end if; end process writeaoutphas;

writeboutphas : process(vclk) begin if vclk'event and vclk = '1' then
 boutphas <= (not hct(9) or not (hct(7) or hct(6) or hct(5))) and not vct(9) and ((vct(8) xor vct(7))
 or (not vct(8) and not vct(7) and vct(6) and vct(5)) or (vct(8) and vct(7) and not vct(6) and not vct(5)));
 end if; end process writeboutphas;
------------------------------------------------------------------------------------------------------------
comboutput  : process(aoutphas,boutphas,pixel(7)) begin

 if aoutphas = '1' then vign <= pixel(7); vird <= pixel(7); vibl <= '0';

 elsif boutphas = '1' then vign <= '0'; vird <= '0'; vibl <= '1';
 else                      vign <= '0'; vird <= '0'; vibl <= '0'; end if;
 end process comboutput;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
writerepdl : process(cl1) begin if cl1'event and cl1 = '1' then repdl <= repk;
end if; end process writerepdl;

accp <= repdl and key;

writerunzeit : process(nrs,cl1) begin if nrs = '0' then runzeit <= '1';
elsif cl1'event and cl1 = '1' then if (cl2 and ((repk and not repdl) or accp)) = '1' then runzeit <= accp;
end if; end if; end process writerunzeit;

writezeit : process(nrs,cl1) begin if nrs = '0' then zeit <= x"ffffffff";
elsif cl1'event and cl1 = '1' then if cl2 = '1' then zeit <= neuzeit;
end if; end if; end process writezeit;

combneuzeit : process(zeit,accp,runzeit)
variable vrunzeit,vaccp  : std_logic_vector(31 downto 0);
variable o               : std_logic; begin

o := accp;     vaccp    := o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o &
                           o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o;
o := runzeit;  vrunzeit := o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o &
                           o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o;

neuzeit <= not vaccp and ((vrunzeit and std_logic_vector(unsigned(zeit)+1)) or (not vrunzeit and zeit));
end process combneuzeit;
------------------------------------------------------------------------------------------------------------
writevzeit : process(vclk) begin
if vclk'event and vclk = '1' then if (wtrx and (vct(8) xor vct(7))) = '1' then vzeit <= zeit;
end if; end if; end process writevzeit;

writexzeit : process(vclk) begin
if vclk'event and vclk = '1' then if zphas = '1' then xzeit <= xneuzeit;
end if; end if; end process writexzeit;

xneuzeit <= xzeit(27 downto 0) & xzeit(31 downto 28) when hcp(7) = '0' else vzeit;

inx <= xzeit(11 downto 8) & not vct(2) & vct(1 downto 0);

writexpix : process(vclk) begin
if vclk'event and vclk = '1' then if zphas = '1' then
xpix <= wert(to_integer(unsigned(inx)));
end if; end if; end process writexpix;

writezpix : process(vclk) begin
if vclk'event and vclk = '1' then if zphas = '1' then zpix <= xpix;
end if; end if; end process writezpix;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
end behvdogen;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity allh is port(

       cl1    : in  std_logic;
       nrs    : in  std_logic;
       dtast  : in  std_logic;
       n0     : in  std_logic;
       n1     : in  std_logic;
       n2     : in  std_logic;
       n3     : in  std_logic;
       ads    : in  std_logic_vector(15 downto 0);
       zeich  : in  std_logic_vector( 7 downto 0);
       datx   : in  std_logic_vector( 7 downto 0);
       hs     : out std_logic;
       adx    : out std_logic_vector(18 downto 1);
       datinh : out std_logic_vector( 7 downto 0);
       vpg    : out std_logic;
       bram   : out std_logic;
       adsgk  : out std_logic;
       adsck  : out std_logic;
       adsrk  : out std_logic);

end allh;
architecture behallh of allh is

signal bufh        : std_logic_vector(7 downto 0);
signal slt         : std_logic_vector(2 downto 0);

signal mmp,mmx,mmy,mmz,
       nnp,nnx,nny,nnz,
       adm         : std_logic_vector(1 downto 0);

signal adsnn,adsmn,adskk,adsen,adsnv,
       adse4,adse5,adse6,adse7,
       wslt,nh,aders            : std_logic;

signal tvpg                     : std_logic := '0';
signal enab4,enab5,enab6,enab7  : std_logic := '0';
signal leer4,leer5,leer6,leer7  : std_logic := '1';

-------------------------------------------------------------------------------------------------------------------
begin
-------------------------------------------------------------------------------------------------------------------
writevpg : process(cl1) begin if cl1'event and cl1 = '0' then if (adsnv and nrs)= '1'
                        then tvpg <= not tvpg; end if; end if; end process writevpg;
vpg <= tvpg;
-------------------------------------------------------------------------------------------------------------------
pbufh : process(adsmn,wslt,slt,ads(2 downto 0),leer4,leer5,leer6,leer7,
                      mmp,mmx,mmy,mmz,nnp,nnx,nny,nnz,dtast,zeich,datx)

variable s2,s1,s0,z2,z1,z0,a3,a2,a1,a0,dd,mm,vv : std_logic;
variable aa3,aa2,aa1,aa0 : std_logic_vector(3 downto 0);  begin

s2 := ads(2); s1 := ads(1); s0 := ads(0);
z2 := not s2; z1 := not s1; z0 := not s0;

a3 := s1 and s0; aa3 := a3 & a3 & a3 & a3; a2 := s1 and z0; aa2 := a2 & a2 & a2 & a2;
a1 := z1 and s0; aa1 := a1 & a1 & a1 & a1; a0 := z1 and z0; aa0 := a0 & a0 & a0 & a0;

mm := adsmn;

bufh <= zeich or datx or

  ((wslt and z2 and a0 and '1'  ) &
   (wslt and z2 and a1 and '0'  ) &
   (wslt and z2 and a2 and '1'  ) &
   (wslt and z2 and a3 and '1'  ) &
   (wslt and s2 and a0 and leer4) &
   (wslt and s2 and a1 and leer5) &
   (wslt and s2 and a2 and leer6) &
   (wslt and s2 and a3 and leer7) )  or

  ((mm & mm & mm & mm & mm & mm & mm & mm) and (dtast & slt &
    ((aa3 and (mmz & nnz)) or (aa2 and (mmy & nny)) or (aa1 and (mmx & nnx)) or (aa0 and (nnp & mmp)))));

end process pbufh;

writedatinh : process(cl1) begin if cl1'event and cl1 = '1' then
              datinh <= bufh;    end if; end process writedatinh;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
nh <= not n3 and not n2 and n1 and n0; hs <= nh;

adsnn <= nh  and not ads(7) and not ads(6) and not ads(5) and     ads(4) ;
wslt  <= nh  and not ads(7) and not ads(6) and not ads(5) and not ads(4) and     ads(3) ;
adsmn <= nh  and not ads(7) and not ads(6) and not ads(5) and not ads(4) and not ads(3) and     ads(2);
adskk <= nh  and not ads(7) and not ads(6) and not ads(5) and not ads(4) and not ads(3) and not ads(2);

adsen <= nh  and not ads(7) and not ads(6) and     ads(5) and not ads(4) and     ads(3) and     ads(2);

adsnv <= adskk and     ads(1) and     ads(0);
adsgk <= adskk and     ads(1) and not ads(0);
adsck <= adskk and not ads(1) and     ads(0);
adsrk <= adskk and not ads(1) and not ads(0);

adse4 <= adsen and not ads(1) and not ads(0);
adse5 <= adsen and not ads(1) and     ads(0);
adse6 <= adsen and     ads(1) and not ads(0);
adse7 <= adsen and     ads(1) and     ads(0);

-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
bram <=  not adm(1) and adm(0) and ads(15);

adm  <=  (((not n1 and not n0) & (not n1 and not n0)) and mmp) or
         (((not n1 and     n0) & (not n1 and     n0)) and mmx) or
         (((    n1 and not n0) & (    n1 and not n0)) and mmy) or
         (((    n1 and     n0) & (    n1 and     n0)) and mmz) ;

adx(18 downto 16) <= "0" & adm when (adm(1) or adm(0) or not slt(2)) = '1' else slt;

adx(15 downto  1) <= ads(15) & ((not adm(1) and adm(0) and ads(15) and tvpg) xor ads(14)) & ads(13 downto 10)

                     & (((aders & aders & aders) and slt) xor ads(9 downto 7)) & ads(6 downto 1);

aders <= adm(1) and not adm(0) and ads(15) and ads(14) and ads(13) and ads(12) and ads(11) and ads(10);
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writeslt : process(cl1) begin if cl1'event and cl1 = '0' then if (wslt and (nnp(1) or nnp(0))) = '1' then
                                 slt <= ads(2) &  ads(1) &  ads(0) ; end if; end if; end process writeslt;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writennp : process(cl1,nrs)  begin if nrs = '0' then nnp <= "10"; elsif cl1'event and cl1 = '0' then
 if (not n3 and n2 and not n1 and not n0) = '1' then nnp <= mmp; end if; end if; end process writennp;

writemmx : process(cl1,nrs)  begin if nrs = '0' then mmx <= "01"; elsif cl1'event and cl1 = '0' then
 if (not n3 and n2 and not n1 and     n0) = '1' then mmx <= nnx; end if; end if; end process writemmx;

writemmy : process(cl1,nrs)  begin if nrs = '0' then mmy <= "01"; elsif cl1'event and cl1 = '0' then
 if (not n3 and n2 and     n1 and not n0) = '1' then mmy <= nny; end if; end if; end process writemmy;

writemmz : process(cl1,nrs)  begin if nrs = '0' then mmz <= "01"; elsif cl1'event and cl1 = '0' then
 if (not n3 and n2 and     n1 and     n0) = '1' then mmz <= nnz; end if; end if; end process writemmz;
-------------------------------------------------------------------------------------------------------------------
writemmp : process(cl1,nrs)  begin if nrs = '0' then mmp <= "01"; elsif cl1'event and cl1 = '0' then
 if ((not n3 and n2 and not n1 and not n0) or (adsnn and not ads(3) and not ads(2))) = '1' then
      mmp <= (nnp and (n2 & n2)) or ((ads(1) & ads(0)) and not (n2 & n2)) ; end if; end if; end process writemmp;

writennx : process(cl1,nrs)  begin if nrs = '0' then nnx <= "10"; elsif cl1'event and cl1 = '0' then
 if ((not n3 and n2 and not n1 and     n0) or (adsnn and not ads(3) and     ads(2))) = '1' then
      nnx <= (mmx and (n2 & n2)) or ((ads(1) & ads(0)) and not (n2 & n2)) ; end if; end if; end process writennx;

writenny : process(cl1,nrs)  begin if nrs = '0' then nny <= "10"; elsif cl1'event and cl1 = '0' then
 if ((not n3 and n2 and     n1 and not n0) or (adsnn and     ads(3) and not ads(2))) = '1' then
      nny <= (mmy and (n2 & n2)) or ((ads(1) & ads(0)) and not (n2 & n2)) ; end if; end if; end process writenny;

writennz : process(cl1,nrs)  begin if nrs = '0' then nnz <= "10"; elsif cl1'event and cl1 = '0' then
 if ((not n3 and n2 and     n1 and     n0) or (adsnn and     ads(3) and     ads(2))) = '1' then
      nnz <= (mmz and (n2 & n2)) or ((ads(1) & ads(0)) and not (n2 & n2)) ; end if; end if; end process writennz;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writeenab4 : process(cl1)  begin if cl1'event and cl1 = '0' then if (adse4 and nrs) = '1'
                           then enab4 <= '1'; end if; end if; end process writeenab4;
writeenab5 : process(cl1)  begin if cl1'event and cl1 = '0' then if (adse5 and nrs) = '1'
                           then enab5 <= '1'; end if; end if; end process writeenab5;
writeenab6 : process(cl1)  begin if cl1'event and cl1 = '0' then if (adse6 and nrs) = '1'
                           then enab6 <= '1'; end if; end if; end process writeenab6;
writeenab7 : process(cl1)  begin if cl1'event and cl1 = '0' then if (adse7 and nrs) = '1'
                           then enab7 <= '1'; end if; end if; end process writeenab7;

writeleer4 : process(cl1)  begin if cl1'event and cl1 = '0' then if ((adse4 and nrs) or (enab4 and not nrs)) = '1'
                           then leer4 <= nrs; end if; end if; end process writeleer4;
writeleer5 : process(cl1)  begin if cl1'event and cl1 = '0' then if ((adse5 and nrs) or (enab5 and not nrs)) = '1'
                           then leer5 <= nrs; end if; end if; end process writeleer5;
writeleer6 : process(cl1)  begin if cl1'event and cl1 = '0' then if ((adse6 and nrs) or (enab6 and not nrs)) = '1'
                           then leer6 <= nrs; end if; end if; end process writeleer6;
writeleer7 : process(cl1)  begin if cl1'event and cl1 = '0' then if ((adse7 and nrs) or (enab7 and not nrs)) = '1'
                           then leer7 <= nrs; end if; end if; end process writeleer7;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
end behallh;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity keybd is port(

  nrs    : in  std_logic;
  cl1    : in  std_logic;
  cl2    : in  std_logic;

  adsgk  : in  std_logic;
  adsck  : in  std_logic;
  adsrk  : in  std_logic;

  ps2c   : in  std_logic;
  ps2d   : in  std_logic;

  repk   : out std_logic;
  dtast  : out std_logic;
  zeich  : out std_logic_vector(7 downto 0));

end keybd;
architecture behkeybd of keybd is

type feld is array(0 to 255) of std_logic_vector(7 downto 0);
constant tabelle : feld := (

x"78",x"78",x"26",x"26",x"78",x"78",x"2a",x"2e",x"7e",x"7e",x"75",x"77",x"7d",x"7d",x"29",x"29",
x"78",x"78",x"27",x"27",x"20",x"20",x"2b",x"2f",x"7f",x"7f",x"79",x"79",x"72",x"72",x"78",x"78",
x"78",x"78",x"70",x"70",x"7b",x"7b",x"78",x"78",x"78",x"78",x"52",x"12",x"2c",x"2c",x"78",x"78",
x"78",x"78",x"78",x"78",x"5a",x"1a",x"54",x"14",x"42",x"02",x"58",x"18",x"2d",x"2d",x"71",x"71",
x"78",x"78",x"44",x"04",x"59",x"19",x"45",x"05",x"46",x"06",x"66",x"6e",x"67",x"6f",x"71",x"71",
x"78",x"78",x"00",x"00",x"57",x"17",x"47",x"07",x"55",x"15",x"53",x"13",x"65",x"6d",x"71",x"71",
x"78",x"78",x"4f",x"0f",x"43",x"03",x"49",x"09",x"48",x"08",x"5b",x"1b",x"64",x"6c",x"78",x"78",
x"78",x"78",x"78",x"78",x"4e",x"0e",x"4b",x"0b",x"56",x"16",x"63",x"6b",x"62",x"6a",x"78",x"78",
x"78",x"78",x"23",x"23",x"4c",x"0c",x"4a",x"0a",x"50",x"10",x"60",x"68",x"61",x"69",x"78",x"78",
x"78",x"78",x"22",x"22",x"21",x"21",x"4d",x"0d",x"5d",x"1d",x"51",x"11",x"24",x"24",x"78",x"78",
x"78",x"78",x"78",x"78",x"5c",x"1c",x"78",x"78",x"5e",x"1e",x"25",x"25",x"78",x"78",x"78",x"78",
x"7a",x"7a",x"40",x"40",x"7c",x"7c",x"5f",x"1f",x"78",x"78",x"41",x"01",x"78",x"78",x"78",x"78",
x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"78",x"73",x"73",x"78",x"78",
x"78",x"78",x"3d",x"3f",x"78",x"78",x"30",x"32",x"3c",x"3e",x"78",x"78",x"78",x"78",x"78",x"78",
x"76",x"76",x"34",x"36",x"39",x"3b",x"78",x"78",x"31",x"33",x"38",x"3a",x"78",x"78",x"78",x"78",
x"28",x"28",x"78",x"78",x"35",x"37",x"78",x"78",x"78",x"78",x"74",x"74",x"78",x"78",x"78",x"78");

signal czro,cset,vale0,valf0,valsh,datneu,datoff   : std_logic;
signal ena,sft,ign,nok,dbufo,key,rep,repdl         : std_logic := '0';

signal cbuf        : std_logic_vector( 3 downto 0) := x"f";
signal cc          : std_logic_vector( 7 downto 0) := x"00";
signal neucc,inccc : std_logic_vector( 7 downto 0);
signal ccmax       : std_logic;

signal dbuf        : std_logic_vector(11 downto 0) := x"000";
signal neudbuf     : std_logic_vector(11 downto 0);
signal cod         : std_logic_vector( 7 downto 0);

signal csct        : std_logic_vector(18 downto 0) := "1100000000000000000";
signal neucsct     : std_logic_vector(18 downto 0);
signal inccsct     : std_logic_vector(18 downto 0);

signal zeit        : std_logic_vector(23 downto 0);
signal neuzeit     : std_logic_vector(23 downto 0);
signal newzeit     : std_logic_vector(23 downto 0);
signal deczeit     : std_logic_vector(23 downto 0);
signal vales,htast : std_logic;

begin
----------------------------------------------------------------------------------------------------------------
repk <= rep;
----------------------------------------------------------------------------------------------------------------
writecbuf : process(cl1) begin
if cl1'event and cl1 = '1' then cbuf <= ps2c & cbuf(3 downto 1);
end if; end process writecbuf;

cset   <= cbuf(3) and cbuf(2) and cbuf(1) and cbuf(0);
czro   <= not (cbuf(3) or cbuf(2) or cbuf(1) or cbuf(0));
----------------------------------------------------------------------------------------------------------------
ccmax <= cc(7) and cc(6);

writecc : process(cl1) begin
if cl1'event and cl1 = '1' then cc <= neucc;
end if; end process writecc;

neucc <= x"00" when cbuf(3) = '0' else inccc;
inccc <= cc when ccmax = '1' else std_logic_vector(unsigned(cc) + 1);
----------------------------------------------------------------------------------------------------------------
writeena : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 and (cset or czro))= '1' then ena <= cset;
end if; end if; end process writeena;
----------------------------------------------------------------------------------------------------------------
writedbuf : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 and (ccmax or (czro and ena))) = '1' then dbuf <= neudbuf;
end if; end if; end process writedbuf;

neudbuf <= x"800" when ccmax = '1' else ps2d & dbuf(11 downto 1);
----------------------------------------------------------------------------------------------------------------
writedbufo : process(cl1) begin
if cl1'event and cl1 = '1' then if cl2 = '1' then dbufo <= dbuf(0);
end if; end if; end process writedbufo;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
vale0 <= dbuf(9) and dbuf(8) and dbuf(7) and not (dbuf(6) or dbuf(5) or dbuf(4) or dbuf(3) or dbuf(2));
valf0 <= dbuf(9) and dbuf(8) and dbuf(7) and dbuf(6) and not (dbuf(5) or dbuf(4) or dbuf(3) or dbuf(2));
valsh <= dbuf(6) and dbuf(4) and not (dbuf(9) or dbuf(8) or dbuf(7) or dbuf(5) or dbuf(3) or dbuf(2));
vales <= dbuf(8) and dbuf(7) and dbuf(6) and dbuf(4) and dbuf(3) and not (dbuf(9) or dbuf(5) or dbuf(2));
----------------------------------------------------------------------------------------------------------------
datneu <= dbuf(0) and not (dbufo or vale0 or valf0 or valsh or vales or ign);
datoff <= dbuf(0) and not dbufo and valf0;
----------------------------------------------------------------------------------------------------------------
writekey : process(nrs,cl1) begin if nrs = '0' then key <= '0'; elsif cl1'event and cl1 = '1' then
if ((cl2 and (datneu or datoff or (repdl and key))) or adsck) = '1' then key <= datneu;
end if; end if; end process writekey;
----------------------------------------------------------------------------------------------------------------
writecod : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 and datneu) = '1' then cod <= dbuf(9 downto 2);
end if; end if; end process writecod;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
writeign : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 = '1') and ((not dbuf(0) or dbufo or vale0) = '0') then
ign <= valf0; end if; end if; end process writeign;
----------------------------------------------------------------------------------------------------------------
writesft : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 = '1') and ((not dbuf(0) or dbufo or vale0 or ign) = '0') then
sft <= valsh or (sft and not (valf0 and nok)); end if; end if; end process writesft;
----------------------------------------------------------------------------------------------------------------
writenok : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 = '1') and ((not dbuf(0) or dbufo or vale0 or ign or valsh) = '0') then
nok <= valf0; end if; end if; end process writenok;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
combzeich : process(adsrk,repdl,cod,sft,key)
variable adrzei : unsigned(7 downto 0);
begin
adrzei := unsigned(cod(6 downto 0) & sft);
if (adsrk or repdl) = '0' then zeich <= x"00";
elsif cod(7) = '1' then zeich <= key & "1111000";
else zeich <= tabelle(to_integer(adrzei)) or (key & "0000000"); end if;
end process combzeich;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
writerep : process(nrs,cl2) begin if nrs = '0' then rep <= '0'; elsif cl2'event and cl2 = '1' then
if (adsgk or (repdl and key) or (csct(18) and csct(17))) = '1' then rep <= adsgk;
end if; end if; end process writerep;

writerepdl : process(cl1) begin if cl1'event and cl1 = '1' then repdl <= rep;
end if; end process writerepdl;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
writecsct : process(cl1) begin
if cl1'event and cl1 = '1' then if cl2 = '1' then csct <= neucsct; end if; end if;
end process writecsct;

neucsct <= "000" & x"0000" when adsgk = '1' else inccsct;

inccsct <= csct when (csct(18) and csct(17)) = '1' else std_logic_vector(unsigned(csct) + 1);
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
dtast <= htast or zeit(23);

writehtast : process(cl1,nrs) begin if nrs = '0' then htast <= '0';
             elsif cl1'event and cl1 = '1' then if (cl2 and repdl and key) = '1' then htast <= zeit(23);
             end if; end if; end process writehtast;

writezeit  : process(cl1,nrs) begin if nrs = '0' then zeit <= x"7fffff";
             elsif cl1'event and cl1 = '1' then if cl2 = '1' then zeit <= neuzeit;
             end if; end if; end process writezeit;

neuzeit <= x"7fffff" when (repdl and key)= '1' else newzeit;
newzeit <= x"ffffff" when (dbuf(0) and not dbufo and vales) = '1' else deczeit;
deczeit <= x"7fffff" when zeit(23) = '0' else std_logic_vector(unsigned(zeit)-1);
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
end behkeybd;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity expa is port(

          nrs   : in  std_logic;
          cl1   : in  std_logic;
          cl2   : in  std_logic;
          hs    : in  std_logic;
          ads   : in  std_logic_vector(15 downto 0);
          datx  : out std_logic_vector( 7 downto 0);
          repx  : out std_logic;

          rxd   : in  std_logic;
          txd   : out std_logic;
          din   : in  std_logic;
          dou   : out std_logic;
          dck   : out std_logic;
          dce   : out std_logic);

end expa;
architecture behexpa of expa is

signal adssi,adsso,adsde,adsdd,adsd1,adsd4,bs : std_logic;
signal datsi,datsd                            : std_logic_vector(7 downto 0);
signal repsi,repsd                            : std_logic;

component sio port(

          nrs   : in  std_logic;
          cl1   : in  std_logic;
          cl2   : in  std_logic;
          adh8  : in  std_logic_vector(7 downto 0);
          dat8  : out std_logic_vector(7 downto 0);
          rep   : out std_logic;

          adssi : in  std_logic;
          adsso : in  std_logic;
          rxd   : in  std_logic;
          txd   : out std_logic);

end component;

component sdc port(

          nrs   : in  std_logic;
          cl1   : in  std_logic;
          cl2   : in  std_logic;
          adh8  : in  std_logic_vector(7 downto 0);
          dat8  : out std_logic_vector(7 downto 0);
          rep   : out std_logic;

          adsdd : in  std_logic;
          adsde : in  std_logic;
          adsd1 : in  std_logic;
          adsd4 : in  std_logic;
          din   : in  std_logic;
          dou   : out std_logic;
          dck   : out std_logic;
          dce   : out std_logic);

end component;
                      
begin

bs    <= hs and not ads(7) and not ads(6) and     ads(5) and not ads(4);

adssi <= bs and not ads(3) and not ads(2) and     ads(1) and not ads(0); -- 22
adsso <= bs and not ads(3) and not ads(2) and     ads(1) and     ads(0); -- 23

adsdd <= bs and     ads(3) and not ads(2) and not ads(1) and not ads(0); -- 28
adsde <= bs and     ads(3) and not ads(2) and not ads(1) and     ads(0); -- 29
adsd1 <= bs and     ads(3) and not ads(2) and     ads(1) and not ads(0); -- 2a
adsd4 <= bs and     ads(3) and not ads(2) and     ads(1) and     ads(0); -- 2b

repx  <= repsi or repsd;
datx  <= datsi or datsd;

isio : sio port map(nrs,cl1,cl2,ads(15 downto 8),datsi,repsi,adssi,adsso,rxd,txd);
isdc : sdc port map(nrs,cl1,cl2,ads(15 downto 8),datsd,repsd,adsdd,adsde,adsd1,adsd4,din,dou,dck,dce);

----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
end behexpa;
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sio is port(

          nrs   : in  std_logic;
          cl1   : in  std_logic;
          cl2   : in  std_logic;
          adh8  : in  std_logic_vector(7 downto 0);
          dat8  : out std_logic_vector(7 downto 0);
          rep   : out std_logic;

          adssi : in  std_logic;
          adsso : in  std_logic;
          rxd   : in  std_logic;
          txd   : out std_logic);

end sio;
architecture behsio of sio is

signal datsi             : std_logic_vector(7 downto 0);
signal repsi,repdl       : std_logic := '0';
signal rxdff             : std_logic := '1';
signal maxz,maxzeit      : std_logic       ;
signal testz             : std_logic_vector(4 downto 0) := "00000";
signal zeit              : std_logic_vector(4 downto 0) := "00000";
signal inbuf             : std_logic_vector(8 downto 0) := "100000000";
signal obf,neobf,vneu    : std_logic_vector(9 downto 0);
signal ozeit             : std_logic_vector(4 downto 0);
signal orun,neu,maxozeit : std_logic;

begin

rep   <= repsi;
dat8  <= datsi;

----------------------------------------------------------------------------------------------------------------------
datsi <= x"00" when repdl = '0' else inbuf(8 downto 1);
----------------------------------------------------------------------------------------------------------------------
writerepsi : process(nrs,cl2) begin if nrs = '0' then repsi <= '0'; elsif cl2'event and cl2 = '1' then
if (adssi or inbuf(0)) = '1' then repsi <= adssi;
end if; end if; end process writerepsi;

writerepdl : process(cl1) begin
if cl1'event and cl1 = '1' then if cl2 = '1' then repdl <= repsi;
end if; end if; end process writerepdl;
----------------------------------------------------------------------------------------------------------------------
writerxdff : process(cl1) begin
if cl1'event and cl1 = '1' then if cl2 = '1' then rxdff <= rxd; end if; end if; end process writerxdff;
----------------------------------------------------------------------------------------------------------------------
maxz <= '1' when testz = "10000" else '0';

writetestz : process(repsi,cl1) begin
if repsi = '0' then testz <= "00000"; elsif cl1'event and cl1 = '1' then
if (cl2 and not maxz and not rxdff) = '1' then testz <= std_logic_vector(unsigned(testz)+1);
end if; end if; end process writetestz;
----------------------------------------------------------------------------------------------------------------------
maxzeit <= '1' when zeit = "11010" else '0';

writezeit : process(repsi,cl1) begin
if repsi = '0' then zeit <= "00000"; elsif cl1'event and cl1 = '1' then if (cl2 and maxz) = '1' then
zeit <= std_logic_vector(unsigned(zeit)+1) and not (maxzeit & maxzeit & maxzeit & maxzeit & maxzeit);
end if; end if; end process writezeit;
----------------------------------------------------------------------------------------------------------------------
writeinbuf : process(repdl,cl1) begin
if repdl = '0' then inbuf <= "100000000"; elsif cl1'event and cl1 = '1' then
if (cl2 and maxzeit) = '1' then inbuf <= rxd & inbuf(8 downto 1);
end if; end if; end process writeinbuf;
----------------------------------------------------------------------------------------------------------------------
-- end si -- begin so
----------------------------------------------------------------------------------------------------------------------
txd <= obf(0);

neu <= adsso; vneu <= neu & neu & neu & neu & neu & neu & neu & neu & neu & neu;

writeorun : process(cl2) begin
if cl2'event and cl2 = '1' then
   if (neu or not (obf(9) or obf(8) or obf(7) or obf(6) or obf(5) or
                             obf(4) or obf(3) or obf(2) or obf(1))) = '1' then orun <= neu;
end if; end if; end process writeorun;

writeobf : process(nrs,cl1) begin
if nrs = '0' then obf  <= '0' & x"00" & '1'; elsif cl1'event and cl1 = '1' then
                                      if (cl2 and orun and maxozeit) = '1' then obf <= neobf;
end if; end if; end process writeobf;

neobf <= (vneu and ('1' & adh8 & '0')) or (not vneu and ('0' & obf(9 downto 1)));
----------------------------------------------------------------------------------------------------------------------
maxozeit <= '1' when ozeit = "11010" else '0';

writeozeit : process(cl1,orun) begin if orun = '0' then ozeit <= "11010";
elsif cl1'event and cl1 = '1' then if cl2 = '1' then ozeit <= std_logic_vector(unsigned(ozeit) + 1)
                           and not (maxozeit & maxozeit & maxozeit & maxozeit & maxozeit);
end if; end if; end process writeozeit;
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
end behsio;
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sdc is port(

          nrs   : in  std_logic;
          cl1   : in  std_logic;
          cl2   : in  std_logic;
          adh8  : in  std_logic_vector(7 downto 0);
          dat8  : out std_logic_vector(7 downto 0);
          rep   : out std_logic;

          adsdd : in  std_logic;
          adsde : in  std_logic;
          adsd1 : in  std_logic;
          adsd4 : in  std_logic;
          din   : in  std_logic;
          dou   : out std_logic;
          dck   : out std_logic;
          dce   : out std_logic);

end sdc;
architecture behsdc of sdc is

signal datsd                  : std_logic_vector(7 downto 0);
signal act,sdc,lang,run1,run4 : std_logic;
signal cnt                    : std_logic_vector(3 downto 0);
signal nobf,oubf,inbf,ctr     : std_logic_vector(7 downto 0);

                      
begin

rep   <= '0';
dat8  <= datsd;

writedce : process(nrs,cl1) begin
if nrs = '0' then dce <= '1'; elsif cl1'event and cl1 = '1' then
if (adsde or adsdd) = '1' then dce <= adsdd; end if; end if;
end process writedce;

datsd <= x"00" when (adsd1 or adsd4) = '0' else inbf;
dou   <= oubf(7);
dck   <= sdc;

writelang : process(cl1) begin
if cl1'event and cl1 = '1' then if (adsd1 or adsd4) = '1' then lang <= adsd4; end if; end if;
end process writelang;

run1 <= adsd1 or (not ctr(7) and not lang);
run4 <= adsd4 or (not ctr(7) and     lang);

nobf <= adh8 when (adsd1 or adsd4) = '1' else oubf(6 downto 0) & '1';

writecnt : process(nrs,cl1) begin
if nrs = '0' then cnt <= "1000"; elsif cl1'event and cl1 = '1' then
if ((run4 or not cnt(3)) and cl2) = '1' then cnt <= cnt(2 downto 0) & cnt(3); end if; end if;
end process writecnt;

writectr : process(nrs,cl1) begin
if nrs = '0' then ctr <= "10000000"; elsif cl1'event and cl1 = '1' then
if (((run4 and cnt(3)) or run1) and cl2) = '1' then ctr <= ctr(6 downto 0) & ctr(7) ; end if; end if;
end process writectr;

writeoubf : process(nrs,cl1) begin
if nrs = '0' then oubf <= "10000000"; elsif cl1'event and cl1 = '1' then
if (((run4 and cnt(3)) or run1) and cl2) = '1' then oubf <= nobf; end if; end if;
end process writeoubf;

writesdc : process(nrs,cl1) begin
if nrs = '0' then sdc <= '1'; elsif cl1'event and cl1 = '1' then
if ((((run4 and cnt(3)) or cnt(1)) and cl2) or (run1 or not (lang or sdc))) = '1' then sdc <= not sdc; end if; end if;
end process writesdc;

writeinbf : process(cl1) begin
if cl1'event and cl1 = '0' then if (sdc and act) = '1' then inbf <= inbf(6 downto 0) & din; end if; end if;
end process writeinbf;

writeact : process(sdc,cl1) begin
if sdc = '0' then act <= '1'; elsif cl1'event and cl1 = '0' then if sdc = '1' then act <= '0'; end if; end if;
end process writeact;
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
end behsdc;
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
