---------------
library altera;
use altera.altera_primitives_components.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity pden is port(

          clk   : in  std_logic;

          btn1  : in  std_logic;
          btn0  : in  std_logic;

          sw3   : in  std_logic;
          sw2   : in  std_logic;
          sw1   : in  std_logic;
          sw0   : in  std_logic;

          sdce  : out std_logic;
          mosi  : out std_logic;
          miso  : in  std_logic;
          sdck  : out std_logic;

          ps2c  : in  std_logic;
          ps2d  : in  std_logic;

          rxd   : in  std_logic;
          txd   : out std_logic;

          led7  : out std_logic;
          led6  : out std_logic;
          led5  : out std_logic;
          led4  : out std_logic;
          led3  : out std_logic;
          led2  : out std_logic;
          led1  : out std_logic;
          led0  : out std_logic;

          vibl  : out std_logic;
          vign  : out std_logic;
          vird  : out std_logic;
          vihs  : out std_logic;
          vivs  : out std_logic;

          dcse  : out std_logic;
          dckp  : out std_logic;
          dcke  : out std_logic;
          dldm  : out std_logic;
          dudm  : out std_logic;
          dras  : out std_logic;
          dcas  : out std_logic;
          dwen  : out std_logic;
          dadr  : out   std_logic_vector(14 downto 0);
          ddat  : inout std_logic_vector(15 downto 0);

          acen  : out   std_logic;
          bcen  : out   std_logic);

end pden;
architecture behpden of pden is

signal phd      : std_logic                     :=          '0';
signal qhd      : std_logic_vector( 7 downto 0) :=   "00000001";
signal ctd      : std_logic_vector( 9 downto 0) := "0000000000";
signal pre      : std_logic_vector( 7 downto 0) := x"00";
signal wrt      : std_logic_vector( 7 downto 0) := x"00";

signal ffb1,ffb0,soft,setr,wst,xx1,xx2,wtrx,
       rd,rfhphas,driphas,rowphas,colphas,wtephas    : std_logic := '0';
signal ini,inidl,tra,trax                            : std_logic := '1';
signal p00,p01,p02,pdn,eras,ecas,ewen,vclk,vclx      : std_logic;

signal datres,datmem,dathaw      : std_logic_vector( 7 downto 0);
signal datinh,datin              : std_logic_vector( 7 downto 0);
signal databuf                   : std_logic_vector(15 downto 0);
signal adx                       : std_logic_vector(18 downto 1);
signal datind                    : std_logic_vector( 7 downto 0);
signal iadr,kadr                 : std_logic_vector(14 downto 0);

signal clx,clz                   : std_logic := '1';
signal nrs,nrsdl                 : std_logic := '0';
signal cl1,cl2,rep,n3,n2,n1,n0   : std_logic;
signal datibus,datobus           : std_logic_vector( 7 downto 0);
signal adribus,adrobus,ads,radr  : std_logic_vector(15 downto 0);
------------------------------------------------------------------------------------------------------
signal vpgd                                : std_logic := '0';
signal vdat                                : std_logic_vector( 7 downto 0);
signal vdomem,vadr                         : std_logic_vector(14 downto 0);
signal nachlad,vpg,vren                    : std_logic;
------------------------------------------------------------------------------------------------------
signal adsgk,adsck,adsrk,bram              : std_logic;
------------------------------------------------------------------------------------------------------
signal hs,dtast,sden                       : std_logic;
signal repx,repk                           : std_logic;
signal datx,zeich                          : std_logic_vector( 7 downto 0);
------------------------------------------------------------------------------------------------------
signal badr                                : std_logic_vector(14 downto 0);
signal bdat                                : std_logic_vector( 7 downto 0);
signal bwen,bren                           : std_logic;
------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
component vdogen port(

  nrs       : in  std_logic;
  cl1       : in  std_logic;
  cl2       : in  std_logic;
  key       : in  std_logic;
  repk      : in  std_logic;
  wtrx      : in  std_logic;
  trax      : in  std_logic;
  vpg       : in  std_logic;
  vclk      : in  std_logic;
  bufpixel  : in  std_logic_vector( 7 downto 0);
  vdomem    : out std_logic_vector(14 downto 0);
  nachlad   : out std_logic;
  vibl      : out std_logic;
  vign      : out std_logic;
  vird      : out std_logic;
  vihs      : out std_logic;
  vivs      : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
component cpu port(

      cl0  : in  std_logic;
      cl1  : in  std_logic;
      nrs  : in  std_logic;
      rep  : in  std_logic;

      n3   : out std_logic;
      n2   : out std_logic;
      n1   : out std_logic;
      n0   : out std_logic;

      datibus : in  std_logic_vector ( 7 downto 0);
      datobus : out std_logic_vector ( 7 downto 0);
      adribus : in  std_logic_vector (15 downto 0);
      adrobus : out std_logic_vector (15 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component ram32 port(

       cclk  : in  std_logic;
       cena  : in  std_logic;
       cadr  : in  std_logic_vector(14 downto 0);
       cdat  : in  std_logic_vector( 7 downto 0);

       vclk  : in  std_logic;
       vena  : in  std_logic;
       vadr  : in  std_logic_vector(14 downto 0);
       vdat  : out std_logic_vector( 7 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component allh port(

       cl1    : in  std_logic;
       nrs    : in  std_logic;
       dtast  : in  std_logic;
       n0     : in  std_logic;
       n1     : in  std_logic;
       n2     : in  std_logic;
       n3     : in  std_logic;
       ads    : in  std_logic_vector(15 downto 0);
       zeich  : in  std_logic_vector( 7 downto 0);
       datx   : in  std_logic_vector( 7 downto 0);
       hs     : out std_logic;
       adx    : out std_logic_vector(18 downto 1);
       datinh : out std_logic_vector( 7 downto 0);
       vpg    : out std_logic;
       bram   : out std_logic;
       adsgk  : out std_logic;
       adsck  : out std_logic;
       adsrk  : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
component keybd port(

       nrs    : in  std_logic;
       cl1    : in  std_logic;
       cl2    : in  std_logic;

       adsgk  : in  std_logic;
       adsck  : in  std_logic;
       adsrk  : in  std_logic;

       ps2c   : in  std_logic;
       ps2d   : in  std_logic;

       repk   : out std_logic;
       dtast  : out std_logic;
       zeich  : out std_logic_vector(7 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component expa port(

       nrs   : in  std_logic;
       cl1   : in  std_logic;
       cl2   : in  std_logic;
       hs    : in  std_logic;
       ads   : in  std_logic_vector(15 downto 0);
       datx  : out std_logic_vector( 7 downto 0);
       repx  : out std_logic;

       rxd   : in  std_logic;
       txd   : out std_logic;
       din   : in  std_logic;
       dou   : out std_logic;
       dck   : out std_logic;
       dce   : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
begin

ic0 : global  port map(vclx,vclk);
ic1 : global  port map(clx ,cl1 );
ic2 : global  port map(clz ,cl2 );

i00 : vdogen  port map(nrs,cl1,cl2,zeich(7),repk,wtrx,trax,vpgd,vclk,
                       vdat,vdomem,nachlad,vibl,vign,vird,vihs,vivs);

i01 : cpu     port map(cl1,cl1,nrs,rep,n3,n2,n1,n0,datibus,datobus,adribus,adrobus);

i02 : ram32   port map(cl1,bwen,badr,datobus,vclk,vren,vadr,vdat);

iah : allh    port map(cl1,nrs,dtast,n0,n1,n2,n3,ads,zeich,datx,
                       hs,adx,datinh,vpg,bram,adsgk,adsck,adsrk);

i03 : keybd   port map(nrs,cl1,cl2,adsgk,adsck,adsrk,ps2c,ps2d,repk,dtast,zeich);

i04 : expa    port map(nrs,cl1,cl2,hs,ads,datx,repx,rxd,txd,miso,mosi,sdck,sden);
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
acen <= '1';
bcen <= '1';

led7 <= dtast;
led6 <= '0';
led5 <= not nrs;
led4 <= rep;
led3 <= not sden;
led2 <= not miso;
led1 <= '0';
led0 <= sw3;

sdce <= sden;

dcke <= pre(7);
dckp <= clk;

ads  <= adrobus;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writepre : process(clk) begin
if clk'event and clk = '0' then if pre(7) = '0' then pre <= std_logic_vector(unsigned(pre)+1);
end if; end if; end process writepre;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writephd : process(clk)
begin
if clk'event and clk = '0' then if pre(7) = '1' then phd <= not phd;
end if; end if; end process writephd;

vclx <= not phd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writeqhd : process(clk)
begin
if clk'event and clk = '0' then if phd = '1' then qhd <= qhd(6 downto 0) & qhd(7); end if; end if;
end process writeqhd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writectd : process(clk)
begin
if clk'event and clk = '0' then if (qhd(7) and phd) = '1'
                           then ctd <= std_logic_vector(unsigned(ctd)+1); end if; end if;
end process writectd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writecl2 : process(clk) begin if clk'event and clk = '0'
                        then if (qhd(4) and phd) = '1' then clz <= not ctd(0);
                        end if; end if; end process writecl2;

writecl1 : process(clk) begin if clk'event and clk ='0'
                        then if ((qhd(1) or qhd(7)) and phd) = '1' then clx <= qhd(7);
                        end if; end if; end process writecl1;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writeini   : process(clk) begin if clk'event and clk = '0'
             then if (pdn and phd) = '1' then ini <= '0'; end if; end if; end process writeini;

writeinidl : process(clk) begin if clk'event and clk = '0'
             then if phd = '1' then inidl <= ini; end if; end if; end process writeinidl;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
combctdphas : process(ctd,qhd(7))
variable s5,s4,s3,s2,s1,s0,z4,z3,z2,z1,z0 : std_logic;
begin
s5 := ctd(9) and ctd(8) and ctd(7) and ctd(6) and ctd(5);

s4 := ctd(4); s3 := ctd(3); s2 := ctd(2); s1 := ctd(1); s0 := ctd(0);
z4 := not s4; z3 := not s3; z2 := not s2; z1 := not s1; z0 := not s0;

p00 <= s5 and z4 and z3 and z2 and z1 and z0 and qhd(7);
p01 <= s5 and z4 and s3                      and qhd(7);
p02 <= s5 and s4 and s3 and z2 and z1 and z0 and qhd(7);
pdn <= s5 and s4 and s3 and s2 and s1 and s0 and qhd(7);

end process combctdphas;
-----------------------------------------------------------------------------------------------------------
eras <= not (rowphas or rfhphas or (ini and (p00 or p01 or p02)));
ecas <= not (colphas or rfhphas or (ini and (       p01 or p02)));
ewen <= not (wtephas            or (ini and (p00        or p02)));
-----------------------------------------------------------------------------------------------------------
writecomphas : process(clk)
begin
if clk'event and clk = '0' then if phd = '1' then
                           rowphas <= not ini and qhd(2) and not ctd(0) and n3;
                           colphas <= not ini and qhd(3) and not ctd(0) and n3;
                           wtephas <= not ini and qhd(3) and not ctd(0) and n3 and not n2 and (n1 or n0);
                           rfhphas    <= not ini and qhd(1) and     ctd(0) ;              end if; end if;
end process writecomphas;
-----------------------------------------------------------------------------------------------------------
writedriphas : process(clk)
begin
if clk'event and clk = '0' then if phd = '1' then
                           driphas <= not ini and (tra or (n3 and not n2 and (n1 or n0)))
                           and (qhd(2) or qhd(3) or qhd(4) or qhd(5)) and not ctd(0);     end if; end if;
end process writedriphas;
-----------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------
writedcse : process(clk) begin if clk'event and clk = '0' then
                         dcse <= phd;
                         end if; end process writedcse;

writecmd  : process(clk) begin if clk'event and clk = '0' then
                         dras <= eras or phd; dcas <= ecas or phd; dwen <= ewen or phd;
                         end if; end process writecmd;

writerd   : process(clk) begin if clk'event and clk = '1' then
                         rd <= qhd(5) and phd;
                         end if; end process writerd;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
iadr(14) <= '0';
iadr(13) <= '0';
iadr(12) <= '0';
iadr(11) <= '0';
iadr(10) <= ini and p00;
iadr(9)  <= '0';
iadr(8)  <= '0';
iadr(7)  <= '0';
iadr(6)  <= '0';
iadr(5)  <= ini and p02;
iadr(4)  <= '0';
iadr(3)  <= '0';
iadr(2)  <= '0';
iadr(1)  <= '0';
iadr(0)  <= ini and p02;
------------------------------------------------------------------------------------------
dadr <= iadr or kadr;
------------------------------------------------------------------------------------------
combkadr : process(rowphas,colphas,adx)
variable   oo                          : std_logic;
variable   vrowphas,vcolphas           : std_logic_vector(14 downto 0);
begin
oo := rowphas; vrowphas := oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo;
oo := colphas; vcolphas := oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo & oo;

kadr <= (vrowphas and ("0000" & adx(15) & adx(13 downto 10) & adx(6 downto 1))) or
        (vcolphas and ("0000100" & adx(18 downto 16) & adx(14) & adx(9 downto 7) & "0"));

end process combkadr;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
dldm <=     ads(0) when ini = '0' else '1';
dudm <= not ads(0) when ini = '0' else '1';

ddat <= datobus & datobus when driphas = '1' else "ZZZZZZZZZZZZZZZZ";
------------------------------------------------------------------------------------------
writedatabuf : process(clk) begin
if clk'event and clk = '1' then if rd = '1' then databuf <= ddat; end if; end if;
end process writedatabuf;

datind <= databuf(15 downto 8) when ads(0)='1' else databuf(7 downto 0);

datin  <= bdat when tra = '1' else datind;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
bwen <= not tra and n3 and not n2 and (n1 or n0) and bram;

badr <= (ads(14) xor vpg) & ads(13 downto 0);
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
vadr <= ads(14 downto 0) when trax = '1' else vdomem;

vren <= bren when trax = '1' else nachlad;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
writexx1  : process(vclk) begin
if vclk'event and vclk = '0' then xx1 <= not ctd(0) and qhd(7);
end if; end process writexx1;

writewtrx : process(vclk) begin
if vclk'event and vclk = '1' then wtrx <= xx1;
end if; end process writewtrx;

writetrax : process(vclk) begin
if vclk'event and vclk = '1' then if wtrx = '1' then trax <= tra;
end if; end if; end process writetrax;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
writexx2  : process(vclk) begin
if vclk'event and vclk = '0' then xx2 <= not ctd(0) and qhd(4);
end if; end process writexx2;

writebren : process(vclk) begin
if vclk'event and vclk = '1' then bren <= xx2;
end if; end process writebren;

writebdat : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 and tra)= '1' then bdat <= vdat;
end if; end if; end process writebdat;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
radr    <= x"4cc0" when tra   = '1' else  x"17e8";
adribus <= radr    when nrsdl = '0' else not radr;

datres  <= wst & "000" & sw3 & sw2 & sw1 & sw0 when nrsdl = '0' else x"00";
datmem  <= datin     when (n3 and (n2 or not (n1 or n0))) = '1' else x"00";
dathaw  <= datinh    when (not n3 and not n2 and n0)      = '1' else x"00";

datibus <= datres or datmem or dathaw;

rep     <= repk or repx;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writevpgd : process(clk) begin if clk'event and clk = '0' then vpgd <= vpg; end if; end process writevpgd;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writetast  : process(cl1) begin if cl1'event and cl1 = '1' then
             ffb1 <= not btn1; ffb0 <= not btn0; end if; end process writetast;

writesoft  : process(cl1) begin if cl1'event and cl1 = '1' then
             soft <= not n3 and not n2 and n1 and not n0; end if; end process writesoft;

writetra   : process(cl1) begin if cl1'event and cl1 = '1' then if soft = '1' then
             tra <= '0'; end if; end if; end process writetra;

writesetr  : process(cl2) begin if cl2'event and cl2 = '0' then
             setr <= ffb0 or soft; end if; end process writesetr;

writewrt   : process(cl1) begin if cl1'event and cl1 = '1' then if (inidl or wrt(7)) = '0' then
             wrt <= std_logic_vector(unsigned(wrt)+1); end if; end if; end process writewrt;

writenrs   : process(cl2,setr) begin if setr = '1' then nrs <= '0';
             elsif cl2'event and cl2 = '1' then if (ffb1 or wrt(6)) = '1' then nrs <= '1';
             end if; end if; end process writenrs;

writewst   : process(cl1) begin if cl1'event and cl1 = '0' then if (nrs and not tra) = '1' then wst <= '1';
             end if; end if; end process writewst;

writenrsdl : process(cl1) begin
             if cl1'event and cl1 = '0' then nrsdl <= nrs; end if; end process writenrsdl;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
end behpden;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
