-------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity pnxs is port(

          clk   : in  std_logic;

          btn3  : in  std_logic;
          btn2  : in  std_logic;
          btn1  : in  std_logic;
          btn0  : in  std_logic;

          sw3   : in  std_logic;
          sw2   : in  std_logic;
          sw1   : in  std_logic;
          sw0   : in  std_logic;

          sdce  : out std_logic;
          mosi  : out std_logic;
          miso  : in  std_logic;
          sdck  : out std_logic;

          ps2c  : in  std_logic;
          ps2d  : in  std_logic;

          rxd   : in  std_logic;
          txd   : out std_logic;

          leda  : out std_logic_vector(3 downto 0);
          ledk  : out std_logic_vector(7 downto 0);

          led7  : out std_logic;
          led6  : out std_logic;
          led5  : out std_logic;
          led4  : out std_logic;
          led3  : out std_logic;
          led2  : out std_logic;
          led1  : out std_logic;
          led0  : out std_logic;

          vibl  : out std_logic_vector(1 downto 0);
          vign  : out std_logic_vector(2 downto 0);
          vird  : out std_logic_vector(2 downto 0);
          vihs  : out std_logic;
          vivs  : out std_logic;

          usbd  : in  std_logic;
          fcen  : out std_logic;

          dclk  : out std_logic;
          dcre  : out std_logic;
          dadv  : out std_logic;
          dcen  : out std_logic;
          dlbm  : out std_logic;
          dubm  : out std_logic;
          doen  : out std_logic;
          dwen  : out std_logic;
          dadr  : out   std_logic_vector(23 downto 1);
          ddat  : inout std_logic_vector(15 downto 0));

end pnxs;
architecture behpnxs of pnxs is

signal phd      : std_logic                    :=        '0';
signal qhd      : std_logic_vector(7 downto 0) := "00000001";
signal ctd      : std_logic                    :=        '0';
signal pre      : std_logic_vector(7 downto 0) := x"00";
signal wrt      : std_logic_vector( 7downto 0) := x"00";

signal ffb3,ffb2,ffb1,ffb0,soft,setr,wst,xx1,xx2,wtrx  : std_logic := '0';
signal ini,inidl,tra,trax                              : std_logic := '1';
------------------------------------------------------------------------------------------------------
signal ncen,nadv,noen,nwen                : std_logic;
signal ddir,vclk,vclx                     : std_logic;
------------------------------------------------------------------------------------------------------
signal datres,datmem,dathaw     : std_logic_vector( 7 downto 0);
signal datinh,datin             : std_logic_vector( 7 downto 0);
signal databuf                  : std_logic_vector(15 downto 0);
signal adx                      : std_logic_vector(18 downto 1);
signal datind                   : std_logic_vector( 7 downto 0);
------------------------------------------------------------------------------------------------------
signal clx,cl2                  : std_logic := '1';
signal nrs,nrsdl                : std_logic := '0';
signal cl1,rep,n3,n2,n1,n0      : std_logic;
signal datibus,datobus          : std_logic_vector( 7 downto 0);
signal adribus,adrobus,ads,radr : std_logic_vector(15 downto 0);
------------------------------------------------------------------------------------------------------
signal vpgd                               : std_logic := '0';
signal vdat                               : std_logic_vector( 7 downto 0);
signal vdomem,vadr                        : std_logic_vector(14 downto 0);
signal nachlad,vpg,viblx,vignx,virdx,vren : std_logic;
------------------------------------------------------------------------------------------------------
signal adsgk,adsck,adsrk,bram             : std_logic;
------------------------------------------------------------------------------------------------------
signal hs,dtast,sden                      : std_logic;
signal repx,repk                          : std_logic;
signal datx,zeich                         : std_logic_vector( 7 downto 0);
------------------------------------------------------------------------------------------------------
signal badr                               : std_logic_vector(14 downto 0);
signal bdat                               : std_logic_vector( 7 downto 0);
signal bwen,bren                          : std_logic;
------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------
component vdogen port(

  nrs       : in  std_logic;
  cl1       : in  std_logic;
  cl2       : in  std_logic;
  key       : in  std_logic;
  repk      : in  std_logic;
  wtrx      : in  std_logic;
  trax      : in  std_logic;
  vpg       : in  std_logic;
  vclk      : in  std_logic;
  bufpixel  : in  std_logic_vector( 7 downto 0);
  vdomem    : out std_logic_vector(14 downto 0);
  nachlad   : out std_logic;
  vibl      : out std_logic;
  vign      : out std_logic;
  vird      : out std_logic;
  vihs      : out std_logic;
  vivs      : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
component cpu port(

      cl0  : in  std_logic;
      cl1  : in  std_logic;
      nrs  : in  std_logic;
      rep  : in  std_logic;

      n3   : out std_logic;
      n2   : out std_logic;
      n1   : out std_logic;
      n0   : out std_logic;

      datibus : in  std_logic_vector ( 7 downto 0);
      datobus : out std_logic_vector ( 7 downto 0);
      adribus : in  std_logic_vector (15 downto 0);
      adrobus : out std_logic_vector (15 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component ram32 port(

       cclk  : in  std_logic;
       cena  : in  std_logic;
       cadr  : in  std_logic_vector(14 downto 0);
       cdat  : in  std_logic_vector( 7 downto 0);

       vclk  : in  std_logic;
       vena  : in  std_logic;
       vadr  : in  std_logic_vector(14 downto 0);
       vdat  : out std_logic_vector( 7 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component allh port(

       cl1    : in  std_logic;
       nrs    : in  std_logic;
       dtast  : in  std_logic;
       n0     : in  std_logic;
       n1     : in  std_logic;
       n2     : in  std_logic;
       n3     : in  std_logic;
       ads    : in  std_logic_vector(15 downto 0);
       zeich  : in  std_logic_vector( 7 downto 0);
       datx   : in  std_logic_vector( 7 downto 0);
       hs     : out std_logic;
       adx    : out std_logic_vector(18 downto 1);
       datinh : out std_logic_vector( 7 downto 0);
       vpg    : out std_logic;
       bram   : out std_logic;
       adsgk  : out std_logic;
       adsck  : out std_logic;
       adsrk  : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
component keybd port(

       nrs    : in  std_logic;
       cl1    : in  std_logic;
       cl2    : in  std_logic;

       adsgk  : in  std_logic;
       adsck  : in  std_logic;
       adsrk  : in  std_logic;

       ps2c   : in  std_logic;
       ps2d   : in  std_logic;

       repk   : out std_logic;
       dtast  : out std_logic;
       zeich  : out std_logic_vector(7 downto 0));

end component;
----------------------------------------------------------------------------------------------------
component expa port(

       nrs   : in  std_logic;
       cl1   : in  std_logic;
       cl2   : in  std_logic;
       hs    : in  std_logic;
       ads   : in  std_logic_vector(15 downto 0);
       datx  : out std_logic_vector( 7 downto 0);
       repx  : out std_logic;

       rxd   : in  std_logic;
       txd   : out std_logic;
       din   : in  std_logic;
       dou   : out std_logic;
       dck   : out std_logic;
       dce   : out std_logic);

end component;
----------------------------------------------------------------------------------------------------
component display port(

       nrs    : in  std_logic;
       cl1    : in  std_logic;
       cl2    : in  std_logic;

       ffh    : in  std_logic;
       key    : in  std_logic;
       repk   : in  std_logic;

       leda   : out std_logic_vector(3 downto 0) ;
       ledk   : out std_logic_vector(7 downto 0));

end component;
----------------------------------------------------------------------------------------------------
begin

ic0 : BUFG    port map (I => vclx,   O => vclk  );
ic1 : BUFG    port map (I => clx,    O => cl1   );

i00 : vdogen  port map(nrs,cl1,cl2,zeich(7),repk,wtrx,trax,vpgd,vclk,
                       vdat,vdomem,nachlad,viblx,vignx,virdx,vihs,vivs);

i01 : cpu     port map(cl1,cl1,nrs,rep,n3,n2,n1,n0,datibus,datobus,adribus,adrobus);

i02 : ram32   port map(cl1,bwen,badr,datobus,vclk,vren,vadr,vdat);

iah : allh    port map(cl1,nrs,dtast,n0,n1,n2,n3,ads,zeich,datx,
                       hs,adx,datinh,vpg,bram,adsgk,adsck,adsrk);

i03 : keybd   port map(nrs,cl1,cl2,adsgk,adsck,adsrk,ps2c,ps2d,repk,dtast,zeich);

i04 : expa    port map(nrs,cl1,cl2,hs,ads,datx,repx,rxd,txd,miso,mosi,sdck,sden);

i05 : display port map(nrs,cl1,cl2,ffb1,zeich(7),repk,leda,ledk);
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
dclk <= '0';
dcre <= '0';
fcen <= '1';

led7 <= dtast;
led6 <= usbd or ffb2 or ffb3;
led5 <= not nrs;
led4 <= rep;
led3 <= not sden;
led2 <= not miso;
led1 <= '0';
led0 <= sw3;

sdce <= sden;

vibl <= viblx & viblx;
vign <= vignx & vignx & vignx;
vird <= virdx & virdx & virdx;

ads  <= adrobus;
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
writepre : process(clk) begin
if clk'event and clk = '0' then if pre(7) = '0' then pre <= std_logic_vector(unsigned(pre)+1);
end if; end if; end process writepre;
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
writephd : process(clk) begin
if clk'event and clk = '0' then if pre(7) = '1' then phd <= not phd;
end if; end if; end process writephd;

vclx <= not phd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writeqhd : process(clk) begin
if clk'event and clk = '0' then if phd = '1' then qhd <= qhd(6 downto 0) & qhd(7);
end if; end if; end process writeqhd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writectd : process(clk) begin
if clk'event and clk = '0' then if (qhd(7) and phd) = '1'
                           then ctd <= not ctd; end if; end if;
end process writectd;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writecl2 : process(clk) begin if clk'event and clk = '0'
                        then if (qhd(4) and phd) = '1' then cl2 <= not ctd;
                        end if; end if; end process writecl2;

writecl1 : process(clk) begin if clk'event and clk ='0'
                        then if ((qhd(1) or qhd(7)) and phd) = '1' then clx <= qhd(7);
                        end if; end if; end process writecl1;
-----------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------
writeini   : process(clk) begin if clk'event and clk = '0' then if (ctd and qhd(7) and phd) = '1'
                          then ini <= '0'; end if; end if; end process writeini;

writeinidl : process(clk) begin if clk'event and clk = '0' then if phd = '1'
                          then inidl <= ini; end if; end if; end process writeinidl;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
writencen : process(clk,qhd(0)) begin if qhd(0) = '1' then ncen <= '0';
                    elsif clk'event and clk = '0' then if (qhd(2) and phd) = '1' then
                    ncen <= not ini and n3;
                    end if; end if; end process writencen;

dcen <= not ncen;
------------------------------------------------------------------------------------------
writenadv : process(clk,qhd(1)) begin if qhd(1) = '1' then nadv <= '0';
                    elsif clk'event and clk = '0' then if (qhd(2) and phd) = '1' then
                    nadv <= not ini and n3;
                    end if; end if; end process writenadv;

dadv <= not nadv;
------------------------------------------------------------------------------------------
writenoen : process(clk,qhd(1)) begin if qhd(1) = '1' then noen <= '0';
                    elsif clk'event and clk = '0' then if (qhd(2) and phd) = '1' then
                    noen <= not ini and not ctd and n3 and (n2 or (not n1 and not n0));
                    end if; end if; end process writenoen;

doen <= not noen;
------------------------------------------------------------------------------------------
writenwen : process(clk,qhd(1)) begin if qhd(1) = '1' then nwen <= '0';
                    elsif clk'event and clk = '0' then if (qhd(2) and phd) = '1' then
                    nwen <= not ini and not ctd and (n3 and not n2 and (n1 or n0));
                    end if; end if; end process writenwen;

dwen <= not nwen;
------------------------------------------------------------------------------------------
dadr <= "00000" & adx;
ddir <= not n2 and (n1 or n0);

dlbm <=     ads(0);
dubm <= not ads(0);

ddat <= datobus & datobus  when ddir = '1' else "ZZZZZZZZZZZZZZZZ";
---------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------
writedatabuf : process(clk) begin if clk'event and clk = '0' then if (noen and qhd(7) and phd) = '1' then
               databuf <= ddat;   end if; end if; end process writedatabuf;

datind <= databuf(15 downto 8) when ads(0)= '1' else databuf(7 downto 0);

datin  <= bdat when tra = '1' else datind;
---------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------
bwen <= not tra and n3 and not n2 and (n1 or n0) and bram;

badr <= (ads(14) xor vpg) & ads(13 downto 0);
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
vadr <= ads(14 downto 0) when trax = '1' else vdomem;

vren <= bren when trax = '1' else nachlad;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
writexx1  : process(vclk) begin
if vclk'event and vclk = '0' then xx1 <= not ctd and qhd(7);
end if; end process writexx1;

writewtrx : process(vclk) begin
if vclk'event and vclk = '1' then wtrx <= xx1;
end if; end process writewtrx;

writetrax : process(vclk) begin
if vclk'event and vclk = '1' then if wtrx = '1' then trax <= tra;
end if; end if; end process writetrax;
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
writexx2  : process(vclk) begin
if vclk'event and vclk = '0' then xx2 <= not ctd and qhd(4);
end if; end process writexx2;

writebren : process(vclk) begin
if vclk'event and vclk = '1' then bren <= xx2;
end if; end process writebren;

writebdat : process(cl1) begin
if cl1'event and cl1 = '1' then if (cl2 and tra)= '1' then bdat <= vdat;
end if; end if; end process writebdat;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
radr    <= x"4cc0" when tra   = '1' else  x"17e8";
adribus <= radr    when nrsdl = '0' else not radr;

datres  <= wst & "000" & sw3 & sw2 & sw1 & sw0 when nrsdl = '0' else x"00";
datmem  <= datin     when (n3 and (n2 or not (n1 or n0))) = '1' else x"00";
dathaw  <= datinh    when (not n3 and not n2 and n0)      = '1' else x"00";

datibus <= datres or datmem or dathaw;

rep     <= repk or repx;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writevpgd : process(clk) begin if clk'event and clk = '0' then vpgd <= vpg; end if; end process writevpgd;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writeffbx  : process(cl1) begin if cl1'event and cl1 = '1' then
             ffb2 <= btn2; ffb3 <= btn3; end if; end process writeffbx;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writetast  : process(cl1) begin if cl1'event and cl1 = '1' then
             ffb1 <= btn1; ffb0 <= btn0; end if; end process writetast;

writesoft  : process(cl1) begin if cl1'event and cl1 = '1' then
             soft <= not n3 and not n2 and n1 and not n0; end if; end process writesoft;

writetra   : process(cl1) begin if cl1'event and cl1 = '1' then if soft = '1' then
             tra <= '0'; end if; end if; end process writetra;

writesetr  : process(cl2) begin if cl2'event and cl2 = '0' then
             setr <= ffb0 or soft; end if; end process writesetr;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
writewrt   : process(cl1) begin if cl1'event and cl1 = '1' then if (inidl or wrt(7)) = '0' then
             wrt <= std_logic_vector(unsigned(wrt)+1); end if; end if; end process writewrt;

writenrs   : process(cl2,setr) begin if setr = '1' then nrs <= '0';
             elsif cl2'event and cl2 = '1' then if (ffb1 or wrt(6)) = '1' then nrs <= '1';
             end if; end if; end process writenrs;

writewst   : process(cl1) begin if cl1'event and cl1 = '0' then if (nrs and not tra) = '1' then wst <= '1';
             end if; end if; end process writewst;

writenrsdl : process(cl1) begin
             if cl1'event and cl1 = '0' then nrsdl <= nrs; end if; end process writenrsdl;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
end behpnxs;
-------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity display is port(

  nrs    : in  std_logic;
  cl1    : in  std_logic;
  cl2    : in  std_logic;

  ffh    : in  std_logic;
  key    : in  std_logic;
  repk   : in  std_logic;

  leda   : out std_logic_vector(3 downto 0) ;
  ledk   : out std_logic_vector(7 downto 0));

end display;
architecture behdisplay of display is

type feld is array(0 to 15) of std_logic_vector(7 downto 0);

constant tabelle : feld := (x"3f",x"06",x"5b",x"4f",x"66",x"6d",x"7d",x"07",
                            x"7f",x"6f",x"77",x"75",x"39",x"57",x"79",x"71");

signal repdl,runzeit,accp,ctrmax          : std_logic;
signal zeit,neuzeit                       : std_logic_vector(31 downto 0);

signal zeitbuf,neuzeitbuf,halbzeit        : std_logic_vector(15 downto 0);

signal ctr                                : std_logic_vector(11 downto 0) := x"000";

signal wert                               : std_logic_vector( 7 downto 0) := x"00";
signal phas                               : std_logic_vector( 3 downto 0) := "0000";
signal neuphas                            : std_logic_vector( 3 downto 0) ;
----------------------------------------------------------------------------------------------------------------
begin

leda <= not phas;
ledk <= not wert;
----------------------------------------------------------------------------------------------------------------
writerepdl : process(cl1) begin if cl1'event and cl1 = '1' then repdl <= repk;
end if; end process writerepdl;

accp <= repdl and key;

writerunzeit : process(nrs,cl1) begin if nrs = '0' then runzeit <= '1';
elsif cl1'event and cl1 = '1' then if (cl2 and ((repk and not repdl) or accp)) = '1' then runzeit <= accp;
end if; end if; end process writerunzeit;

writezeit : process(nrs,cl1) begin if nrs = '0' then zeit <= x"ffffffff";
elsif cl1'event and cl1 = '1' then if cl2 = '1' then zeit <= neuzeit;
end if; end if; end process writezeit;

combneuzeit : process(zeit,accp,runzeit)
variable vrunzeit,vaccp  : std_logic_vector(31 downto 0);
variable o               : std_logic; begin

o := accp;     vaccp    := o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o &
                           o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o;
o := runzeit;  vrunzeit := o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o &
                           o & o & o & o & o & o & o & o & o & o & o & o & o & o & o & o;

neuzeit <= not vaccp and ((vrunzeit and std_logic_vector(unsigned(zeit)+1)) or (not vrunzeit and zeit));
end process combneuzeit;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
writectr : process(cl1) begin
if cl1'event and cl1 = '1' then if cl2 = '1' then ctr <= std_logic_vector(unsigned(ctr)+1);
end if; end if; end process writectr;

ctrmax <= ctr(11) and ctr(10) and ctr(9) and ctr(8) and ctr(7) and ctr(6) and ctr(5) and ctr(4)
                                                    and ctr(3) and ctr(2) and ctr(1) and ctr(0) and cl2;
----------------------------------------------------------------------------------------------------------------
neuphas <= phas(0) & phas(3 downto 1) when (phas(3) or phas(2) or phas(1) or phas(0)) = '1' else "0001";

writephas    : process(cl1) begin if cl1'event and cl1 = '1' then if ctrmax = '1' then
phas <= neuphas;                                              end if; end if; end process writephas;

halbzeit <= zeit(31 downto 16) when ffh = '1' else zeit(15 downto 0);

neuzeitbuf <= halbzeit when phas(1) = '1' else zeitbuf(11 downto 0) & "0000";

writezeitbuf : process(cl1) begin if cl1'event and cl1 = '1' then if ctrmax = '1' then
zeitbuf <= neuzeitbuf;                                        end if; end if; end process writezeitbuf;

writewert    : process(cl1) begin if cl1'event and cl1 = '1' then if ctrmax = '1' then
wert <= tabelle(to_integer(unsigned(zeitbuf(15 downto 12)))); end if; end if; end process writewert;
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
end behdisplay;
---------------
---------------
